`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Liam Duffy
// 2017
//////////////////////////////////////////////////////////////////////////////////

module VGAUnique(
    input clk,
    input rst,
    input wire [10:0] row, col,
    output reg [3:0] red, green, blue );

localparam [0:167] mask = 168'h618C63619E7761B37F61B36B61BF637DB3637DB363;

wire [3:0] rmod[1:3];
wire [4:0] cmod[1:3];
wire vis[1:3];

reg [11:0] frame = 12'd0;
reg [10:0] fcol[1:3], frow[1:3];

genvar i;
generate
    for (i=1; i<4; i=i+1) begin
	assign cmod[i] = (fcol[i] >> i+1) % 32;
	assign rmod[i] = (frow[i] >> i+1) % 16;
	assign vis[i] = ((rmod[i] < 7) && (cmod[i] < 25)) ?
		mask[rmod[i]*24 + cmod[i]] : 1'd0;
    end
endgenerate

integer j;
always@ (posedge clk, posedge rst)
    if (rst) begin
	for (j=1; j<4; j=j+1)
	    fcol[j] <= 11'd0;

	for (j=1; j<4; j=j+1)
	    frow[j] <= 11'd0;

	frame <= 12'd0;
	red   <= 4'd0;
	green <= 4'd0;
	blue  <= 4'd0;

    end else begin
	if ((row == 479) && (col == 639))
	    frame <= frame + 1;

	for (j=1; j<4; j=j+1)
	    fcol[j] <= col - (frame >> j);

	for (j=1; j<4; j=j+1)
	    frow[j] <= row - (frame >> j);

	red   <= vis[3] ? 4'hF : 4'h0;
	green <= vis[2] ? 4'hF : 4'h0;
	blue  <= vis[1] ? 4'hF : 4'h0;
    end

endmodule
